/***************************************************************************
 GETMODEL.C - Routines to determine model types
 
 Written by Lee Liming, 6/6/92
 
 NOTE: The code in this module is based on many heuristics.  It is not 
       guaranteed to work forever, and will probably break in the future 
       when new OS releases or hardware types are introduced.

 Copyright (C) 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include "../lib/netuse.h"


/* #define TESTING */


#ifdef sun

/***************************************************************************
 The following code is derived from the table on page CPU-13 of the Sun
 Field Engineer Handbook, Technical Volume 1.  At present, it only gives
 five possible return types: 0 (unknown), MODEL_SUN3 (unspecified Sun 3),
 MODEL_SUN4 (unspecified Sun 4), MODEL_SUN4_65 (SPARCstation 1+), and 
 MODEL_SUN4_50 (SPARCstation IPX).
***************************************************************************/

#include <stdio.h>
#include <sys/types.h>


#ifdef __STDC__
u_char modelDetect(void)
#else
u_char modelDetect()
#endif
{
     unsigned int hostid;

     hostid=gethostid();
     if (hostid<0x02000000) return(0);
     if (hostid<0x21000000) return(MODEL_SUN3);
     if (hostid<0x31000000) return(MODEL_SUN4);
     if (hostid<0x41000000) return(0);
     if (hostid<0x51000000) return(MODEL_SUN3);
     if (hostid<0x53000000) return(MODEL_SUN4);
     if (hostid<0x54000000) return(MODEL_SUN4_65);
     if (hostid<0x57000000) return(MODEL_SUN4);
     if (hostid<0x58000000) return(MODEL_SUN4_50);
     if (hostid<0x72000000) return(MODEL_SUN4);
     return(0);
}


#endif
#ifdef _AIX

/***************************************************************************
 The following code uses the uname -m command and an unpublished table of
 values to obtain model information for IBM RS/6000 workstations.  This
 may prove somewhat less reliable than the Sun method...
***************************************************************************/

#include <stdio.h>


#define UNAME  "/bin/uname -m"


#ifdef __STDC__
u_char modelDetect(void)
#else
u_char modelDetect()
#endif
{
     char uname[128];
     FILE *inf;
     int modelnum;

     if ((inf=popen(UNAME,"r"))==NULL) {
#ifdef TESTING
          fprintf(stderr,"Unable to execute popen() command.\n");
#endif
          return(0);
     }
     if (fgets(uname,sizeof(uname),inf)==NULL) {
#ifdef TESTING
          fprintf(stderr,"Unable to read uname command's output.\n");
#endif
          return(0);
     }
     else if (uname[strlen(uname)-1]=='\n') uname[strlen(uname)-1]='\0';
     pclose(inf);
     if (strlen(uname)<10) {
#ifdef TESTING
          fprintf(stderr,"Unable to get model id from uname command's output.\n");
#endif
          return(0);
     }
     modelnum=(uname[8]-'0')*0x10+(uname[9]-'0');
#ifdef TESTING
     printf("Model number value is %02x.\n",modelnum);
#endif
     switch (modelnum) {
          case 0x10: return(MODEL_RS530);  /* Could also be 730... */
          case 0x11:
          case 0x14: return(MODEL_RS540);
          case 0x18: return(MODEL_RS530H);
          case 0x1C: return(MODEL_RS550);
          case 0x20: return(MODEL_RS930);
          case 0x2E: return(MODEL_RS950);  /* Could also be 950E... */
          case 0x30: return(MODEL_RS520);
          case 0x31: return(MODEL_RS320);
          case 0x35: return(MODEL_RS320H);
          case 0x37: return(MODEL_RS340);
          case 0x38: return(MODEL_RS350);
          case 0x41: return(MODEL_RS220);
          case 0x5C: return(MODEL_RS560);
     }
     return(0);
}


#endif
#if (defined(apollo) || defined(hpux))

/***************************************************************************
 The following code uses the uname -m command to obtain model information
 for HP/Apollo workstations.
***************************************************************************/

#include <stdio.h>


#ifdef apollo
#define UNAME  "/usr/local/bin/uname -m"
#else
#define UNAME  "/bin/uname -m"
#endif


#ifdef __STDC__
u_char modelDetect(void)
#else
u_char modelDetect()
#endif
{
     char uname[128];
     FILE *inf;

     if ((inf=popen(UNAME,"r"))==NULL) {
#ifdef TESTING
          fprintf(stderr,"Unable to execute popen() command.\n");
#endif
          return(0);
     }
     if (fgets(uname,sizeof(uname),inf)==NULL) {
#ifdef TESTING
          fprintf(stderr,"Unable to read uname command's output.\n");
#endif
          return(0);
     }
     else if (uname[strlen(uname)-1]=='\n') uname[strlen(uname)-1]='\0';
     pclose(inf);
     if (!strcmp(uname,"DN2500") || !strcmp(uname,"DSP2500"))
          return(MODEL_AP2500);
     if (!strcmp(uname,"DN3000") || !strcmp(uname,"DSP3000"))
          return(MODEL_AP3000);
     if (!strcmp(uname,"DN3500") || !strcmp(uname,"DSP3500"))
          return(MODEL_AP3500);
     if (!strcmp(uname,"DN4000") || !strcmp(uname,"DSP4000"))
          return(MODEL_AP4000);
     if (!strcmp(uname,"DN4500") || !strcmp(uname,"DSP4500"))
          return(MODEL_AP4500);
     if (!strcmp(uname,"DN5500") || !strcmp(uname,"DSP5500"))
          return(MODEL_AP5500);
     if (!strcmp(uname,"DN10010") || !strcmp(uname,"DSP10010"))
          return(MODEL_AP10010);
     if (!strcmp(uname,"DN10020") || !strcmp(uname,"DSP10020"))
          return(MODEL_AP10020);
     if (!strcmp(uname,"425T") || !strcmp(uname,"425t")) 
          return(MODEL_HP425T);
     if (!strcmp(uname,"425E") || !strcmp(uname,"425e")) 
          return(MODEL_HP425E);
     if (!strcmp(uname,"9000/705")) return(MODEL_HP9000705);
     if (!strcmp(uname,"9000/710")) return(MODEL_HP9000710);
     if (!strcmp(uname,"9000/720")) return(MODEL_HP9000720);
     if (!strcmp(uname,"9000/730")) return(MODEL_HP9000730);
     if (!strcmp(uname,"9000/750")) return(MODEL_HP9000750);
     return(0);
}


#endif
#ifdef mips

/***************************************************************************
 The following code uses the DEC uerf command to determine the model type
 of a DECstations based on the processor type.  This is one of our sicker
 heuristics, and will probably fail more often...
***************************************************************************/

#include <stdio.h>
#include "../lib/parser.h"

#define UERF  "/usr/etc/uerf -r 300 -R -H %s"


typedef struct _procmapping {
             char   procname[16];
             char   modelname[16];
             u_char model;
        } PROCMAPPING;


PROCMAPPING pmap[]={{"KN01",      "",   MODEL_DS3100},
                    {"KN02-BA",   "120",MODEL_DS5000120},
                    {"KN02-BA",   "133",MODEL_DS5000133},
                    {"KN02/R3000","",   MODEL_DS5000200},
                    {"?",         "",   0}};


#ifdef __STDC__
u_char modelDetect(void)
#else
u_char modelDetect()
#endif
{
     char processor[128],myhostname[128],uerfcommand[128],temp[128],temp2[128];
     FILE *inf;
     int found=0;
     PROCMAPPING *pmptr;

#ifdef TESTING
     fprintf(stderr,"Entering modelDetect().\n");
#endif
     gethostname(myhostname,sizeof(myhostname));
     myhostname[sizeof(myhostname)-1]='\0';
     myhostname[12]='\0';
#ifdef TESTING
     fprintf(stderr,"myhostname is set.\n");
#endif
     sprintf(uerfcommand,UERF,myhostname);
#ifdef TESTING
     fprintf(stderr,"uerfcommand=%s\n",uerfcommand);
#endif
     if ((inf=popen(uerfcommand,"r"))==NULL) {
#ifdef TESTING
          fprintf(stderr,"Unable to execute popen() command.\n");
#endif
          return(0);
     }
     while ((found!=2) && (fgets(processor,sizeof(processor),inf)!=NULL)) {
          if (processor[strlen(processor)-1]=='\n') 
               processor[strlen(processor)-1]='\0';
#ifdef TESTING
          printf("\"%s\"\n",processor);
#endif
          if ((found==1) && !strcmp(textparam(processor,0,temp2),"DECstation")) {
               if (strlen(textparam(processor,3,temp2))) {
                    for (pmptr=pmap; pmptr->model; pmptr++)
                         if (!strcmp(pmptr->procname,temp) &&
                             !strcmp(pmptr->modelname,temp2)) break;
                    found=2;
               }
               continue;
          }
          if (!found && !strcmp(textparam(processor,0,temp),"PROCESSOR")) {
               if (strlen(textparam(processor,2,temp))) {
                    for (pmptr=pmap; 
                         pmptr->model && strcmp(pmptr->procname,temp);
                         pmptr++)
                         ;
                    if (strlen(pmptr->modelname)) found=1;
                    else found=2;
               }
               continue;
          }
     }
     if (found!=2) {
#ifdef TESTING
          fprintf(stderr,"Unable to get processor type from uerf's output.\n");
#endif
          return(0);
     }
     pclose(inf);
     return(pmptr->model);
}


#endif
#if !(defined(sun) || defined(apollo) || defined(mips) || defined(hpux) || defined(_AIX))

#ifdef __STDC__
u_char modelDetect(void)
#else
u_char modelDetect()
#endif
{
     return(0);
}

#endif


#ifdef TESTING

main(argc,argv)
int argc;
char *argv[];
{
     printf("modelDetect() returns %d.\n",modelDetect());
}

#endif
