/******************************************************************************
 NETWORK.C - NETUSE Client-side Network Interface

 This file contains the basic network routines used by the NETUSE clients for
 sending and receiving data from the server.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#ifndef MSDOS
#include <sys/time.h>
#endif
#ifdef IBMTCPIP
#include <types.h>
#endif
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <setjmp.h>
#include "protocol.h"


/* #define DEBUG */


int sock;                               /* Socket used for UDP send/receives */
struct sockaddr_in servername,clientname;           /* Server & client names */
jmp_buf jumpTimeout;                              /* Timeout handler address */
long netuseTimeout=TIMEOUT;                        /* Override timeout value */


/******************************************************************************
 int packetSend(USEREC *msg)

 This function sends a NETUSE packet to the server.  netSetup() must have been
 previously called.  Transmit failures are returned (interpreted) to the
 caller.  All numeric fields in the packet are converted to network order
 before sending.
******************************************************************************/

#ifdef __STDC__
int packetSend(USEREC *msg)
#else
int packetSend(msg)
USEREC *msg;
#endif
{
     u_char chk;
     int rv,count,i;
     PACKET out;

     msg->load1=htons(msg->load1);
     msg->load2=htons(msg->load2);
     msg->load3=htons(msg->load3);
     msg->users=htons(msg->users);
     msg->console=htons(msg->console);
     msg->tmp=htonl(msg->tmp);
     msg->uid=htons(msg->uid);
     bcopy((char *)msg,(char *)out,sizeof(PACKET));
     for (chk=0,i=1; i<PACKET_SIZE; i++)
          chk=(chk+out[i]) % 0xFF;
     out[0]=chk;
#ifdef DEBUG
     printf("NETUSE Client sending:\n");
     printf("Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x Users=%04x Console=%04x /tmp=%08x Mach=%02x Model=%02x Chksum=%02x\n",
            msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
            ntohs(msg->load2),ntohs(msg->load3),ntohs(msg->users),
            ntohs(msg->console),ntohl(msg->tmp),msg->machine,msg->model,out[0]);
     printf("Data:");
     for (i=0; i<PACKET_SIZE; i++) {
          if (i && !(i % 16)) printf("\n     ");
          printf(" %02x",out[i]);
     }
     printf("\n");
#endif
     count=0;
     do {
          rv=sendto(sock,out,PACKET_SIZE,0,
                    &servername,sizeof(struct sockaddr));
          if (rv==(-1)) {
               perror("sendto");
               if (++count>SEND_RETRIES) return(RV_nBADSEND);
          }
     } while (rv!=PACKET_SIZE);
     return(RV_nOK);
}


/******************************************************************************
 int packetReceive(USEREC *msg)

 This function receives a packet from the NETUSE server.  If no packet arrives
 within one timeout period, RV_nTIMEOUT is returned.  Packets with bad
 checksum fields are ignored.  Otherwise, either a network error is returned,
 or the msg buffer is filled with the received packet's data (all numeric
 fields are converted to host format).
******************************************************************************/

#ifdef __STDC__
int packetReceive(USEREC *msg)
#else
int packetReceive(msg)
USEREC *msg;
#endif
{
     u_char chk;
     int i,rv;
     PACKET in;
#ifndef IBMTCPIP
     struct fd_set fdset;
     struct timeval tv;
#else
     int fdset[2];
     long tv;
#endif

     do {
#ifdef IBMTCPIP
          fdset[0]=sock;
          tv=netuseTimeout*1000;
#ifdef DEBUG
          printf("About to call IBM's lobotomized select() in packetReceive().\n");
          fflush(stdout);
#endif
          rv=select(fdset,1,0,0,tv);
#else /* !IBMTCPIP */
          FD_ZERO(&fdset);
          FD_SET(sock,&fdset);
          tv.tv_sec=netuseTimeout;
          tv.tv_usec=0;
#ifdef DEBUG
          printf("About to call select() in packetReceive().\n");
          fflush(stdout);
#endif
          rv=select(FD_SETSIZE,&fdset,NULL,NULL,&tv);
#endif /* !IBMTCPIP */
#ifdef DEBUG
          printf("Back from select() in packetReceive().\n");
          fflush(stdout);
#endif
          if (rv==(-1)) {
               fprintf(stderr,"packetReceive(): Error %d from select().\n",rv);
               return(RV_nBADRECV);
          }
#ifdef IBMTCPIP
               if (fdset[0]!=(-1)) {
#else
               if (FD_ISSET(sock,&fdset)) {
#endif
               i=sizeof(struct sockaddr);
               if (recvfrom(sock,in,PACKET_SIZE,0,
                            &servername,&i)==(-1)) {
                    perror("recvfrom");
                    return(RV_nBADRECV);
               }
               bcopy((char *)in,(char *)msg,sizeof(PACKET));
          }
          else return(RV_nTIMEOUT);
#ifdef DEBUG
          printf("NETUSE Client received:\n");
          printf("Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x Users=%04x Console=%04x /tmp=%08x Mach=%02x Model=%02x Chksum=%02x\n",
                 msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
                 ntohs(msg->load2),ntohs(msg->load3),ntohs(msg->users),
                 ntohs(msg->console),ntohl(msg->tmp),msg->machine,msg->model,in[0]);
          printf("Data:");
          for (i=0; i<PACKET_SIZE; i++) {
               if (i && !(i % 16)) printf("\n     ");
               printf(" %02x",in[i]);
          }
          printf("\n");
#endif
          for (chk=0,i=1; i<PACKET_SIZE; i++)
               chk=(chk+in[i]) % 0xFF;
#ifdef DEBUG
          if (chk!=in[0]) printf("Bad checksum!!!\n");
#endif
     } while (chk!=in[0]);
     msg->load1=ntohs(msg->load1);
     msg->load2=ntohs(msg->load2);
     msg->load3=ntohs(msg->load3);
     msg->users=ntohs(msg->users);
     msg->console=ntohs(msg->console);
     msg->tmp=ntohl(msg->tmp);
     msg->uid=ntohs(msg->uid);
     return(RV_nOK);
}


#ifdef __STDC__
char *textIPAddress(u_long inaddr,char *buf)
#else
char *textIPAddress(inaddr,buf)
u_long inaddr;
char *buf;
#endif
{
     struct hostent *hp;
     u_char ipbytes[4];
     char *cp;

#if (defined(MSDOS) && defined(NOVELL))
     if ((cp=raddr(inaddr))==(char *)(-1)) {
          bcopy((char *)&inaddr,ipbytes,sizeof(ipbytes));
          sprintf(buf,"%d.%d.%d.%d",ipbytes[0],ipbytes[1],ipbytes[2],ipbytes[3]);
     }
     else strcpy(buf,cp);
#else
     hp=gethostbyaddr(&inaddr,sizeof(u_long),AF_INET);
     if (hp==NULL) {
          bcopy((char *)&inaddr,ipbytes,sizeof(ipbytes));
          sprintf(buf,"%d.%d.%d.%d",ipbytes[0],ipbytes[1],ipbytes[2],ipbytes[3])
;
     }
     else strcpy(buf,hp->h_name);
#endif
     return(buf);
}


/******************************************************************************
 void netSetup(void)

 Set up the client side of a NETUSE session.  Get a socket, fill in the
 server's adddress, bind to a local port, etc.
******************************************************************************/

#ifdef __STDC__
void netSetup(void)
#else
void netSetup()
#endif
{
     struct hostent *hp;
     char buf[132],*cp,**cpp;
     long iaddr;

#ifdef DEBUG
     printf("Getting a socket...\n");
#endif
     sock=socket(AF_INET,SOCK_DGRAM,0);
     if (sock<0) {
          perror("Acquiring a socket");
          exit(1);
     }
     clientname.sin_family=AF_INET;
     clientname.sin_addr.s_addr=INADDR_ANY;
     clientname.sin_port=0;
#ifdef DEBUG
     printf("Binding socket...\n");
#endif
     if (bind(sock,&clientname,sizeof(clientname))) {
          perror("Binding socket");
          exit(1);
     }
#ifdef DEBUG
     printf("Resolving server IP address...\n");
#endif
#if (defined(MSDOS) && defined(NOVELL))
     cp=NETUSE_SERVER;
     cpp=(&cp);
     if ((iaddr=rhost(cpp))==(-1)) {
          perror("Getting server address");
          exit(1);
     }
     else bcopy((char *)&iaddr,(char *)&servername.sin_addr,sizeof(iaddr));
#else
     hp=gethostbyname(NETUSE_SERVER);
     if (hp==NULL) {
          perror("Getting server address");
          exit(1);
     }
     bcopy((char *)hp->h_addr,(char *)&servername.sin_addr,hp->h_length);
#endif
#ifdef DEBUG
     printf("Server address = %s\n",
            textIPAddress(servername.sin_addr.s_addr,buf));
#endif
     servername.sin_family=AF_INET;
     servername.sin_port=htons(NETUSE_PORT);
}


/******************************************************************************
 void netShutdown(void)

 Shut down the client side of the NETUSE session.  Close the local socket.
******************************************************************************/

#ifdef __STDC__
void netShutdown(void)
#else
void netShutdown()
#endif
{
#if (defined(NOVELL) || defined(IBMTCPIP))
     soclose(sock);
#else
     close(sock);
#endif
}
