/******************************************************************************
 PARSER.C - String parsing functions

 (c) 1990, 1991, 1992, R. Lee Liming, Jr.
 USED WITH PERMISSION OF THE AUTHOR
 ALL RIGHTS RESERVED

 This file provides basic string manipulation functions for general-purpose
 parsing.

 strip() removes trailing spaces from a string.
 strip_leading() removes leading spaces from a string.
 scrunch() removes all spaces from a string.
 stoupper() converts an entire string to uppercase.
 spicture() puts a string into a fixed-size text field.
 whitespace() returns true if it's argument is a whitespace character,
   false otherwise.
 textparam() extracts the nth token (separated by whitespace, starting at 0)
   from its first argument.
 limits() parses a number range string and returns the start and end values.
 strstr() returns a pointer to a substring within another string.
******************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>

#include "parser.h"


/*****************************************************************************
 char *strip(str)

 This function removes trailing spaces from the string passed to it, and
 returns a pointer to its argument.
*****************************************************************************/

#ifdef __STDC__
char *strip(char *s)
#else
char *strip(s)
char *s;
#endif
{
     char *p;

     for (p=s+strlen(s)-1; (p>=s) && (*p==' '); p--)
          *p='\0';
     return(s);
}


/*****************************************************************************
 char *strip_leading(str)

 This function removes leading spaces from the string passed to it, and
 returns a pointer to its argument.
*****************************************************************************/

#ifdef __STDC__
char *strip_leading(char *s)
#else
char *strip_leading(s)
char *s;
#endif
{
     char *p;

     for (p=s; (*p!='\0') && (*p==' '); p++)
          ;
     strcpy(s,p);
     return(s);
}


/*****************************************************************************
 char *scrunch(char *str)

 This function removes ALL spaces from a string, including internal spaces.
*****************************************************************************/

#ifdef __STDC__
char *scrunch(char *str)
#else
char *scrunch(str)
char *str;
#endif
{
     char *p,*q;

     for (p=str; (*p!='\0'); p++)
          if (*p==' ') {
               for (q=p; (*q==' '); q++)
                    ;
               strcpy(p,q);
          }
     return(str);
}


/*****************************************************************************
 char *stoupper(char *str)

 This function converts the string passed to it to uppercase, and returns a
 pointer to it.
*****************************************************************************/

#ifdef __STDC__
char *stoupper(char *s)
#else
char *stoupper(s)
char *s;
#endif
{
     char *p;

     for (p=s; *p!='\0'; p++) if (islower(*p)) *p=toupper(*p);
     return(s);
}


/*****************************************************************************
 char *spicture(char *str,int n,char *temp)

 This function left-justifies a string in a fixed-size text field of size n.
 The string is padded or truncated as necessary.
*****************************************************************************/

#ifdef __STDC__
char *spicture(char *str,int n,char *temp)
#else
char *spicture(str,n,temp)
char *str;
int n;
char *temp;
#endif
{
     char *p,*q;
     int i;

     for (p=str,q=temp,i=0; i<n; i++)
          if ((p==NULL) || (*p=='\0')) *(q++)=' ';
          else *(q++)=(*(p++));
     *q='\0';
     return(temp);
}


/*****************************************************************************
 int whitespace(ch)

 This function returns true if its argument is a whitespace character (or
 comma), false otherwise.  Used for separating tokens in a string.
*****************************************************************************/

#ifdef __STDC__
int whitespace(char ch)
#else
int whitespace(ch)
char ch;
#endif
{
     switch (ch) {
          case ' ':
          case '\t':
          case ',':
               return(1);
          default:
               return(0);
     }
}


/*****************************************************************************
 char *textparam(str,n,buf)

 This is an interesting function, which breaks the first string up into
 tokens (separated by whitespace(), defined above), and returns the nth one
 in the third parameter.  n starts at 0 (zero) for the first token on the
 line.  The function returns a pointer to the extracted token.  Yeah, it's
 got a lot of +'s and ='s, but it seems to work...  ;-)
*****************************************************************************/

#ifdef __STDC__
char *textparam(char *s,int n,char t[])
#else
char *textparam(s,n,t)
char *s;
int n;
char t[];
#endif
{
     int i;
     char *p;

     for (p=s; (*p!='\0') && whitespace(*p); p++)
          ;
     for (i=0; (i<n) && (*p!='\0'); i++) {
          if (*p=='"') {
               for (p++; (*p!='\0') && (*p!='"'); p++)
                    if ((*p=='\\') && (*(p+1)!='\0')) p++;
               if (*p=='"') p++;
          }
          else for ( ; (*p!='\0') && !whitespace(*p); p++)
                    ;
          for ( ; (*p!='\0') && whitespace(*p); p++)
               ;
     }
     if (*p=='"') {
          for (i=0,p++; (*p!='\0') && (*p!='"'); p++,i++)
               if ((*p=='\\') && (*(p+1)!='\0')) t[i]=(*(++p));
               else t[i]=(*p);
     }
     else for (i=0; (*p!='\0') && !whitespace(*p); p++,i++)
               t[i]=(*p);
     t[i]='\0';
     return(t);
}


/*****************************************************************************
 char *encapsulate(char *str,char *buf)

 The encapsulate() function encapsulates a string into a format that can be
 quoted and then successfully parsed by the textparam() function.  It escapes
 all double quotes with a backslash, and doubles all backslashes in the
 original string.  The breakout() function is used to break the string out
 of this encapsulation.
*****************************************************************************/

#ifdef __STDC__
char *encapsulate(char *str,char *buf)
#else
char *encapsulate(str,buf)
char *str;
char *buf;
#endif
{
     char *p,*q;

     for (p=str,q=buf; *p!='\0'; p++) {
          switch (*p) {
               case '"':
                    *(q++)='\\';
                    *(q++)='"';
                    break;
               case '\\':
                    *(q++)='\\';
                    *(q++)='\\';
                    break;
               default:
                    *(q++)=(*p);
                    break;
          }
     }
     *q='\0';
     return(buf);
}


/*****************************************************************************
 char *breakout(char *str,char *buf)

 The breakout() function is used to break the string out of an encapsulated
 string format made using the encapsulate() function.
*****************************************************************************/

#ifdef __STDC__
char *breakout(char *str,char *buf)
#else
char *breakout(str,buf)
char *str;
char *buf;
#endif
{
     char *p,*q;

     for (p=str,q=buf; *p!='\0'; p++) {
          switch (*p) {
               case '\\':
                    *(q++)=(*(++p));
                    break;
               default:
                    *(q++)=(*p);
                    break;
          }
     }
     *q='\0';
     return(buf);
}


/*****************************************************************************
 int limits(char *rstr,int *start,int *stop)

 The limits function parses a range string of the form "<num> [ - <num> ]"
 into a start and stop value.  If the range syntax is legal, the return
 value is zero.
*****************************************************************************/

#ifdef __STDC__
int limits(char *rstr,int *start,int *stop)
#else
int limits(rstr,start,stop)
char *rstr;
int *start,*stop;
#endif
{
     char *p,temp[10];
     int i;

     strip(strip_leading(rstr));
     if (!strlen(rstr)) return(-1);
     for (p=rstr,i=0; (*p>='0') && (*p<='9'); p++,i++)
          temp[i]=(*p);
     temp[i]='\0';
     if ((*p!='\0') && (*p!='-') && (*p!=' ')) return(-1);
     (*stop)=(*start)=atoi(temp);
     if (*p=='\0') return(0);
     strcpy(temp,++p);
     strip_leading(temp);
     if (temp[0]=='-') {
          strcpy(temp,temp+1);
          strip_leading(temp);
     }
     for (p=temp,i=0; (*p>='0') && (*p<='9'); p++,i++)
          ;
     if (*p!='\0') return(-2);
     *stop=atoi(temp);
     return(0);
}


/*****************************************************************************
 char *strstr(source,target)

 This function searches for the first occurence of the target string in the
 source string, and returns a pointer to it within the source string.  The
 function returns NULL if no match is found.
*****************************************************************************/

#ifdef __STDC__
char *strstr(char *p,char s[])
#else
char *strstr(p,s)
char *p,s[];
#endif
{
     char *idx,*tmp;
     int i,found=0;

     if (!strlen(p)) return(NULL);
     while (!found) {
          if ((tmp=idx=index(p,s[0]))==0) break;
          else {
               for (i=0; (s[i]!='\0') && (*idx==s[i]); idx++,i++)
                    ;
               if (s[i]=='\0') found=1;
               else p=(++tmp);
          }
     }
     if (!found) return(NULL);
     else return(tmp);
}
