/******************************************************************************
 PROTOCOL.H - NETUSE Protocol Definition

 This header file defines the NETUSE protocol, used for communication with
 the NETUSE server (netuse).  All clients, including the netused daemon, use
 this protocol to send and receive information to/from the server.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#ifndef _PROTOCOL_INCLUDED_
#define _PROTOCOL_INCLUDED_

#include <sys/types.h>
#include "config.h"


#define SEND_RETRIES       2   /* Send tries before conceding a send failure */
#define REQ_RETRIES        2     /* Request tries before conceding a failure */
#define TIMEOUT            5    /* Time between request and response timeout */

#define OP_NOP             0                      /* NETUSE protocol opcodes */
#define OP_STATUS          1                        /* netused status report */
#define OP_QUERY           2               /* Request a host of a given type */
#define OP_GETLIST         3                 /* Request the entire host list */
#define OP_DISPLIST        4              /* Request a host list for display */
#define OP_GETDOWN         5         /* Request list of hosts which are down */
#define OP_ADDHOST         6                   /* Add a host to the database */
#define OP_DELHOST         7              /* Remove a host from the database */
#define OP_SAVESTATE       8          /* Store the database in a static file */
#define OP_LOADHOSTS       9     /* Re-read the static file to get host list */
#define OP_TEXTSTATUS     10              /* netused report on users & class */
#define OP_TEXTSTATUS2    11      /* netused report (cont.) on users & class */
#define OP_GETMLIST       12     /* Request the entire host list with models */
#define OP_DISPMLIST      13  /* Request a host list for display with models */
#define OP_GETMDOWN       14       /* Request models of hosts which are down */

#define RV_OK              0     /* Protocol response: All's well            */
#define RV_ACK             1     /* Protocol response: Report acknowledged   */
#define RV_NACK            2     /* Protocol response: Report not ack'ed     */
#define RV_TIMEOUT         3     /* Protocol response: Timeout               */
#define RV_NOMATCH         4     /* Protocol response: Nothing matched       */
#define RV_NOFILE          5     /* Protocol response: Unable to access file */

#define RV_nOK             0     /* Network response: All's well             */
#define RV_nTIMEOUT       -1     /* Network response: Timeout                */
#define RV_nBADSEND       -2     /* Network response: Unable to send packet  */
#define RV_nBADRECV       -3     /* Network response: Unable to receive      */

#define MODE_DISP       0x00        /* Normal listing (limited # of machines */
#define MODE_ALL        0x01     /* List ALL hosts (even ones that are down) */
#define MODE_DOWN       0x02                /* List only hosts that are down */
#define MODE_MASK       0x03                      /* Mask for all list modes */
#define MODE_MODELS     0x04 /* List model types instead of last report time */
#define MODE_NET        0x80                /* Network listing mode (add \r) */

#define MACH_DEC           1                                /* Machine types */
#define MACH_SUN           2
#define MACH_IBM_RS6000    3
#define MACH_APOLLO        4
#define MACH_HP            5

#define MODEL_DS3100       1                          /* Machine model types */
#define MODEL_DS5000200    2
#define MODEL_DS5000120    3
#define MODEL_SUN4         4
#define MODEL_SUN4_65      5
#define MODEL_SUN4_50      6
#define MODEL_RS320        7
#define MODEL_RS520        8
#define MODEL_RS530        9
#define MODEL_RS540       10
#define MODEL_RS730       11
#define MODEL_RS930       12
#define MODEL_AP3000      13
#define MODEL_AP3500      14
#define MODEL_AP4000      15
#define MODEL_AP4500      16
#define MODEL_AP5500      17
#define MODEL_AP10010     18
#define MODEL_AP10020     19
#define MODEL_HP425E      20
#define MODEL_HP425T      21
#define MODEL_HP9000720   22
#define MODEL_SUN3        23
#define MODEL_AP2500      24
#define MODEL_DS5000133   25
#define MODEL_HP9000705   26
#define MODEL_HP9000710   27
#define MODEL_HP9000730   28
#define MODEL_HP9000750   29
#define MODEL_RS220       30
#define MODEL_RS320H      31
#define MODEL_RS340       32
#define MODEL_RS350       33
#define MODEL_RS530H      34
#define MODEL_RS550       35
#define MODEL_RS560       36
#define MODEL_RS950       37

#ifdef mips
#define MACHINETYPE MACH_DEC                  /* Program is running on a DEC */
#endif

#ifdef hpux
#define MACHINETYPE MACH_HP                   /* Program is running on an HP */
#endif

#ifdef sun
#define MACHINETYPE MACH_SUN                  /* Program is running on a Sun */
#endif

#ifdef _IBMR2
#define MACHINETYPE MACH_IBM_RS6000      /* Program is running on an RS/6000 */
#endif

#ifdef apollo
#define MACHINETYPE MACH_APOLLO           /* Program is running on an Apollo */
#endif


typedef struct _userec {
             u_char  chksum;               /* Checksum (for error detection) */
             u_char  opcode;                                /* Packet opcode */
             u_char  retcode;                    /* Return code from request */
             u_char  ack;                            /* Acknowedgement field */
             u_short load1,load2,load3;                     /* Load averages */
             u_short users;                      /* Number of login sessions */
             u_long  tmp;                              /* Free space in /tmp */
             u_short console;         /* Number of login sessions on display */
             u_short uid;                   /* User ID (for making requests) */
             u_short filler1;
             u_char  machine;                             /* Type of machine */
             u_char  model;                                 /* Machine model */
        } USEREC;                             /* A NETUSE packet as a struct */

#define ipaddr tmp                                   /* Packet field aliases */
#define portno load1

typedef u_char PACKET[sizeof(USEREC)];  /* Used for treating packet as array */

#define PACKET_SIZE     sizeof(PACKET)           /* Size (bytes) of a packet */


#endif   /* _PROTOCOL_INCLUDED_ */
