/*****************************************************************************
 VARIABLE.C - The variable management module

 Lee Liming, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 


 This module defines the data structures and operations for constructing
 a table of variable bindings.  This is used for variable storage/retrieval
 in the runtime module.

 varHashName() puts the name of a variable through the hashing algorithm
   to get a bucket number for storing the variable in.
 varAllocate() allocates memory for a variable binding and initializes the
   fields to known starting values.
 varFreeTable() deallocates the entire hash table of variable bindings.
 varSet() sets a variable to a new value, creating the variable if necessary.
 varAdd() adds a variable to the table.
 varDelete() deletes a variable from the table.
 varFind() gets the value of a variable.
 varSaveState() saves the state of the variable table to a file.
 varLoadHosts() empties the variable table and reinitializes it using
   variable names stored in a static file.
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <time.h>
#include <sys/types.h>

#include "protocol.h"
#include "netuse.h"
#include "config.h"
#include "parser.h"
#include "variable.h"


extern long lastSave;

VARTABLE  variables;                           /* The global variable table */
TREE     *theTree;
long      lTime=0;


/*****************************************************************************
 int varHashName(char *name)

 This function hashes the name sent to it and returns a bucket number.
*****************************************************************************/

#ifdef __STDC__
int varHashName(u_long name)
#else
int varHashName(name)
u_long name;
#endif
{
     int i,hash;
     u_char ipbytes[4];

     bcopy(&name,ipbytes,sizeof(ipbytes));
     for (i=hash=0; i<4; i++)
          hash=(hash+ipbytes[i]) % VAR_ENTRIES;
     return(hash);
}


/*****************************************************************************
 USER *userAllocate()

 This function allocates memory for another user node and sets the fields to
 known, useful starting values.
*****************************************************************************/

#ifdef __STDC__
USER *userAllocate(void)
#else
USER *userAllocate()
#endif
{
     USER *userNew;

     userNew=(USER *)malloc(sizeof(USER));
     userNew->name=userNew->tty=NULL;
     userNew->next=NULL;
     return(userNew);
}


/*****************************************************************************
 void userFree(USER *uptr)

 This function deallocates the memory used by a user node.
*****************************************************************************/

#ifdef __STDC__
void userFree(USER *uptr)
#else
void userFree(uptr)
USER *uptr;
#endif
{
     if (uptr->name!=NULL) free(uptr->name);
     if (uptr->tty!=NULL) free(uptr->tty);
     free(uptr);
}


/*****************************************************************************
 void userFreeList(USER *uptr)

 This function deallocates an entire user list.
*****************************************************************************/

#ifdef __STDC__
void userFreeList(USER *uptr)
#else
void userFreeList(uptr)
USER *uptr;
#endif
{
     USER *utmp;

     while (uptr!=NULL) {
          utmp=uptr->next;
          userFree(uptr);
          uptr=utmp;
     }
}


/*****************************************************************************
 USERLIST userCopyList(USERLIST uptr)

 This function makes a copy of the user list pointed to by uptr and returns a
 pointer to the new copy.
*****************************************************************************/

#ifdef __STDC__
USERLIST userCopyList(USERLIST uptr)
#else
USERLIST userCopyList(uptr)
USERLIST uptr;
#endif
{
     USER *utmp,*up;

     utmp=NULL;
     while (uptr!=NULL) {
          if (utmp==NULL) utmp=up=userAllocate();
          else {
               up->next=userAllocate();
               up=up->next;
          }
          if (uptr->name!=NULL) {
               up->name=(char *)malloc(strlen(uptr->name)+1);
               strcpy(up->name,uptr->name);
          }
          if (uptr->tty!=NULL) {
               up->tty=(char *)malloc(strlen(uptr->tty)+1);
               strcpy(up->tty,uptr->tty);
          }
          uptr=uptr->next;
     }
     return(utmp);
}


/*****************************************************************************
 VARENTRY *varAllocate()

 This function allocates memory for another variable binding and sets the
 fields to known, useful starting values.
*****************************************************************************/

#ifdef __STDC__
VARENTRY *varAllocate(void)
#else
VARENTRY *varAllocate()
#endif
{
     VARENTRY *varNew;

     varNew=(VARENTRY *)malloc(sizeof(VARENTRY));
     varNew->name=varNew->class=NULL;
     varNew->ip_addr=0;
     varNew->l1=varNew->l2=varNew->l3=0.0;
     varNew->logtime=0;
     varNew->tmp=0;
     varNew->machine=0;
     varNew->model=0;
     varNew->lastused=0;
     varNew->console=varNew->users=0;
     varNew->userFirst=NULL;
     return(varNew);
}


/*****************************************************************************
 varFree()

 This function frees the memory used by a variable table entry.
*****************************************************************************/

#ifdef __STDC__
void varFree(VARENTRY *vptr)
#else
void varFree(vptr)
VARENTRY *vptr;
#endif
{
     if (vptr->name!=NULL) free(vptr->name);
     if (vptr->userFirst!=NULL) userFreeList(vptr->userFirst);
     free(vptr);
}


/*****************************************************************************
 varFreeTable()

 This function frees the entire variable table, removing all variable
 bindings.
*****************************************************************************/

#ifdef __STDC__
void varFreeTable(void)
#else
void varFreeTable()
#endif
{
     VARENTRY *vptr;
     int i;

     for (i=0; i<VAR_ENTRIES; i++)
          while (variables[i]!=NULL) {
               vptr=variables[i]->next;
               varFree(variables[i]);
               variables[i]=vptr;
          }
}


/*****************************************************************************
 varSet(VARENTRY *var)

 This function sets a variable's value, creating a new variable if necessary.
*****************************************************************************/

#ifdef __STDC__
void varSet(VARENTRY *var)
#else
void varSet(var)
VARENTRY *var;
#endif
{
     int hashval;
     VARENTRY *vptr,*follower;

     if (var==NULL) return;
     hashval=varHashName(var->ip_addr);
     vptr=variables[hashval];
     follower=NULL;
     while ((vptr!=NULL) && (vptr->ip_addr!=var->ip_addr)) {
          follower=vptr;
          vptr=vptr->next;
     }
     if (vptr==NULL) {
          vptr=varAllocate();
          vptr->next=NULL;
          vptr->name=(char *)malloc(strlen(var->name)+1);
          strcpy(vptr->name,var->name);
          vptr->ip_addr=var->ip_addr;
          if (follower!=NULL) follower->next=vptr;
          else variables[hashval]=vptr;
     }
     else if (strcmp(vptr->name,var->name)) {
          free(vptr->name);
          vptr->name=(char *)malloc(strlen(var->name)+1);
          strcpy(vptr->name,var->name);
     }
     vptr->l1=var->l1;                               /* Update simple fields */
     vptr->l2=var->l2;
     vptr->l3=var->l3;
     vptr->logtime=time(0);
     vptr->tmp=var->tmp;
     vptr->machine=var->machine;
     vptr->model=var->model;
     vptr->console=var->console;
     vptr->users=var->users;
     if (var->class!=NULL) {                           /* Update class field */
          if (vptr->class!=NULL) {
               if (!strcmp(vptr->class,var->class)) {
                    free(vptr->class);
                    vptr->class=(char *)malloc(strlen(var->class));
                    strcpy(vptr->class,var->class);
               }
          }
          else {
               vptr->class=(char *)malloc(strlen(var->class));
               strcpy(vptr->class,var->class);
          }
     }
     else if (vptr->class!=NULL) {
          free(vptr->class);
          vptr->class=NULL;
     }
     if (var->userFirst!=NULL) {
          userFreeList(vptr->userFirst);
          vptr->userFirst=userCopyList(var->userFirst);
     }
}


/*****************************************************************************
 void varAdd(VARENTRY *var)

 This function adds a variable to the list, using the ipaddr field as a key,
 and corrects the name field if the variable already exists with a different
 name.
*****************************************************************************/

#ifdef __STDC__
void varAdd(VARENTRY *var)
#else
void varAdd(var)
VARENTRY *var;
#endif
{
     int hashval;
     VARENTRY *vptr,*follower;

     if (var==NULL) return;
     hashval=varHashName(var->ip_addr);
     vptr=variables[hashval];
     follower=NULL;
     while ((vptr!=NULL) && (vptr->ip_addr!=var->ip_addr)) {
          follower=vptr;
          vptr=vptr->next;
     }
     if (vptr==NULL) {
          vptr=varAllocate();
          vptr->next=NULL;
          vptr->name=(char *)malloc(strlen(var->name)+1);
          strcpy(vptr->name,var->name);
          vptr->ip_addr=var->ip_addr;
          if (follower!=NULL) follower->next=vptr;
          else variables[hashval]=vptr;
          vptr->machine=var->machine;
          vptr->model=var->model;
          vptr->console=var->console;
          vptr->users=var->users;
          vptr->logtime=0;
     }
     else if (strcmp(vptr->name,var->name)) {
          free(vptr->name);
          vptr->name=(char *)malloc(strlen(var->name)+1);
          strcpy(vptr->name,var->name);
     }
}


/*****************************************************************************
 void varDelete(VARENTRY *var)

 This function deletes a variable from the list, doing nothing if the
 variable does not exist.
*****************************************************************************/

#ifdef __STDC__
void varDelete(VARENTRY *var)
#else
void varDelete(var)
VARENTRY *var;
#endif
{
     int hashval;
     VARENTRY *vptr,*follower;

     if (var==NULL) return;
     hashval=varHashName(var->ip_addr);
     vptr=variables[hashval];
     follower=NULL;
     while ((vptr!=NULL) && (vptr->ip_addr!=var->ip_addr)) {
          follower=vptr;
          vptr=vptr->next;
     }
     if (vptr!=NULL) {
          if (follower==NULL) variables[hashval]=vptr->next;
          else follower->next=vptr->next;
          varFree(vptr);
     }
}


/*****************************************************************************
 VARENTRY *varFindMatch(VARENTRY *template)

 This function searches the variable table looking for a variable whose value
 matches the criteria given in the template, and returns a pointer to an
 entry which satisfies the criteria.  If no match can be made, the return
 value is NULL.
*****************************************************************************/

#ifdef __STDC__
VARENTRY *varFindMatch(VARENTRY *template)
#else
VARENTRY *varFindMatch(template)
VARENTRY *template;
#endif
{
     int hashval,found;
     VARENTRY *vptr;
     long nowtime;

     time(&nowtime);
     for (found=0,hashval=0; (hashval<VAR_ENTRIES) && !found; hashval++)
          for (vptr=variables[hashval]; (vptr!=NULL) && !found;
               vptr=vptr->next) {
               if ((vptr->l1<=template->l1) && (vptr->l2<=template->l2) &&
                   (vptr->l3<=template->l3) &&
                   (varUsers(vptr)<=varUsers(template)) &&
                   (varConsole(vptr)<=varConsole(template)) &&
                   (vptr->tmp>=template->tmp) &&
                   (!template->machine || (vptr->machine==template->machine)) &&
                   (!template->model || (vptr->model==template->model)) &&
                   ((nowtime-vptr->logtime)<15*60) &&
                   ((nowtime-vptr->lastused)>2*60)) {
                    found=1;
                    vptr->lastused=nowtime;
                    break;
               }
          }
     if (!found) return(NULL);
     else return(vptr);
}


/*****************************************************************************
 VARENTRY *varFind(u_long ip_addr)

 This function looks up an entry in the variable table which has the given
 IP address.  A pointer to the matching table entry is returned.  If no match
 is found, a NULL value is returned.
*****************************************************************************/

#ifdef __STDC__
VARENTRY *varFind(u_long ip_addr)
#else
VARENTRY *varFind(ip_addr)
u_long ip_addr;
#endif
{
     int hashval;
     VARENTRY *vptr;

     hashval=varHashName(ip_addr);
     vptr=variables[hashval];
     while ((vptr!=NULL) && (vptr->ip_addr!=ip_addr))
          vptr=vptr->next;
     return(vptr);
}


#ifdef __STDC__
int varSaveState(void)
#else
int varSaveState()
#endif
{
     FILE *outf;
     char temp[32],temp2[16];
     VARENTRY *vptr;
     int i;

     if ((outf=fopen(NETUSEHOSTFILE,"w"))==NULL) {
          perror(NETUSEHOSTFILE);
          return(RV_NOFILE);
     }
     for (i=0; i<VAR_ENTRIES; i++) {
          for (vptr=variables[i]; vptr!=NULL; vptr=vptr->next)
               fprintf(outf,"%-7s %s %s\n",
                       netGetMachType(vptr->machine,temp),vptr->name,
                       netGetModelType(vptr->model,temp2));
     }
     fclose(outf);
     lastSave=time(0);
     return(RV_OK);
}


#ifdef __STDC__
int varLoadHosts(void)
#else
int varLoadHosts()
#endif
{
     FILE *inf;
     char temp[128],inbuf[512];
     VARENTRY *ventry;

     if ((inf=fopen(NETUSEHOSTFILE,"r"))==NULL) {
          perror(NETUSEHOSTFILE);
          return(RV_NOFILE);
     }
     varFreeTable();
     ventry=varAllocate();
     while (fgets(inbuf,sizeof(inbuf),inf)!=NULL) {
          inbuf[strlen(inbuf)-1]='\0';
          if (!strlen(inbuf) || (inbuf[0]=='#')) continue;
          if (!(ventry->ip_addr=netGetAddress(textparam(inbuf,1,temp))))
               continue;
          ventry->name=(char *)malloc(strlen(temp)+1);
          strcpy(ventry->name,temp);
          if (!strcmp(textparam(inbuf,0,temp),"Sun"))
               ventry->machine=MACH_SUN;
          else if (!strcmp(temp,"DEC"))
               ventry->machine=MACH_DEC;
          else if (!strcmp(temp,"IBM"))
               ventry->machine=MACH_IBM_RS6000;
          else if (!strcmp(temp,"Apo"))
               ventry->machine=MACH_APOLLO;
          else if (!strcmp(temp,"HP"))
               ventry->machine=MACH_HP;
          else ventry->machine=0;
          ventry->model=netStrToModel(textparam(inbuf,2,temp));
          varAdd(ventry);
          free(ventry->name);
     }
     varFree(ventry);
     fclose(inf);
     lastSave=time(0);
     return(RV_OK);
}


#ifdef __STDC__
void varInit(void)
#else
void varInit()
#endif
{
     int i;

     for (i=0; i<VAR_ENTRIES; i++)
          variables[i]=NULL;
     varLoadHosts();
}


#ifdef __STDC__
int varConsole(VARENTRY *vptr)
#else
int varConsole(vptr)
VARENTRY *vptr;
#endif
{
/*
     USER *uptr;

     if ((vptr->userFirst==NULL) && (vptr->console!=0)) return(1);
     for (uptr=vptr->userFirst; uptr!=NULL; uptr=uptr->next)
          if (!strcmp(uptr->tty,"cons")) return(1);
     return(0);
*/
     return(vptr->console);
}


#ifdef __STDC__
int varUsers(VARENTRY *vptr)
#else
int varUsers(vptr)
VARENTRY *vptr;
#endif
{
/*
     USER *uptr;
     int i;

     if ((vptr->userFirst==NULL) && (vptr->users!=0)) return(vptr->users);
     for (i=0,uptr=vptr->userFirst; uptr!=NULL; uptr=uptr->next)
          ++i;
     return(i);
*/
     return(vptr->users);
}


#ifdef __STDC__
char *fmt_time(long theTime,char *s2)
#else
char *fmt_time(theTime,s2)
long theTime;
char *s2;
#endif
{
     struct tm *tim;

     tim=localtime(&theTime);
     sprintf(s2,"%02d/%02d %2d:%02d%s",tim->tm_mon+1,tim->tm_mday,
                ((tim->tm_hour % 12) ? (tim->tm_hour % 12) : 12),
                tim->tm_min,(tim->tm_hour>=12 ? "pm" : "am"));
     return(s2);
}


#ifdef __STDC__
void varDumpTable(char *fname)
#else
void varDumpTable(fname)
char *fname;
#endif
{
     VARENTRY *vptr;
     int i;
     FILE *outf;
     char temp[25],temp2[25],temp3[5];

     if ((outf=fopen(fname,"w"))==NULL) {
          perror("fopen");
          return;
     }
     lTime=time(0);
     fprintf(outf,"Last written: %s\n",ctime(&lTime));
     fprintf(outf,"Host name            Type    Load Averages     Ses Disp Last Report   /tmp    \n");
     fprintf(outf,"-------------------- ---- -------------------- --- ---- ------------- --------\n");
     for (i=0; i<VAR_ENTRIES; i++)
          for (vptr=variables[i]; vptr!=NULL; vptr=vptr->next)
               fprintf(outf,"%20s %-4s %6.2f %6.2f %6.2f %3d %s  %s %8lu\n",
                            spicture(vptr->name,20,temp),
                            netGetMachType(vptr->machine,temp3),
                            vptr->l1,vptr->l2,vptr->l3,varUsers(vptr),
                            (varConsole(vptr) ? "YES" : "NO "),
                            fmt_time(vptr->logtime,temp2),vptr->tmp);
     fclose(outf);
     chmod(fname,0644);
}


#ifdef __STDC__
void varDumpTableWithModels(char *fname)
#else
void varDumpTableWithModels(fname)
char *fname;
#endif
{
     VARENTRY *vptr;
     int i;
     FILE *outf;
     char temp[25],temp2[25],temp3[5];

     if ((outf=fopen(fname,"w"))==NULL) {
          perror("fopen");
          return;
     }
     lTime=time(0);
     fprintf(outf,"Last written: %s\n",ctime(&lTime));
     fprintf(outf,"Host name           Type and Model         Load Averages     Ses Disp /tmp    \n");
     fprintf(outf,"------------------- ------------------- -------------------- --- ---- --------\n");
     for (i=0; i<VAR_ENTRIES; i++)
          for (vptr=variables[i]; vptr!=NULL; vptr=vptr->next)
               fprintf(outf,"%19s %-4s%-15s %6.2f %6.2f %6.2f %3d %s  %8lu\n",
                            spicture(vptr->name,19,temp),
                            netGetMachType(vptr->machine,temp3),
                            netGetModelType(vptr->model,temp2),
                            vptr->l1,vptr->l2,vptr->l3,varUsers(vptr),
                            (varConsole(vptr) ? "YES" : "NO "),vptr->tmp);
     fclose(outf);
     chmod(fname,0644);
}


#define binval(x) (x ? 1 : 0)


#ifdef __STDC__
void treeInsert(TREE **tptr,VARENTRY *vptr)
#else
void treeInsert(tptr,vptr)
TREE **tptr;
VARENTRY *vptr;
#endif
{
     if (*tptr==NULL) {
          (*tptr)=(TREE *)malloc(sizeof(TREE));
          (*tptr)->data=vptr;
          (*tptr)->left=NULL;
          (*tptr)->right=NULL;
     }
     else if ((varConsole(vptr)<varConsole((*tptr)->data)) ||
              ((varConsole(vptr)==varConsole((*tptr)->data)) &&
               (vptr->l2<((*tptr)->data->l2))))
               treeInsert(&((*tptr)->left),vptr);
          else treeInsert(&((*tptr)->right),vptr);
}


#ifdef __STDC__
void treeBuild(u_char machtype,u_char modeltype)
#else
void treeBuild(machtype,modeltype)
u_char machtype,modeltype;
#endif
{
     VARENTRY *vptr;
     int i;

     theTree=NULL;
     for (i=0; i<VAR_ENTRIES; i++)
          for (vptr=variables[i]; vptr!=NULL; vptr=vptr->next)
               if ((!machtype || (machtype==vptr->machine)) &&
                   (!modeltype || (modeltype==vptr->model)))
                    treeInsert(&theTree,vptr);
}


#ifdef __STDC__
void treePrintHost(FILE *outf,TREE *tptr)
#else
void treePrintHost(outf,tptr)
FILE *outf;
TREE *tptr;
#endif
{
     char temp[132],temp2[132],temp3[132];

     if (tptr->data->logtime==0) {
          fprintf(outf,"%20s %-4s ------ ------ ------ --- ---  ------------- --------\n\r",
                  spicture(tptr->data->name,20,temp),
                  netGetMachType(tptr->data->machine,temp3));
     }
     else if ((time(NULL)-tptr->data->logtime)>=DOWNTIME*60) {
          fprintf(outf,"%20s %-4s ------ ------ ------ --- ---  %s --------\n\r",
                  spicture(tptr->data->name,20,temp),
                  netGetMachType(tptr->data->machine,temp3),
                  fmt_time(tptr->data->logtime,temp2));
     }
     else {
          fprintf(outf,"%20s %-4s %6.2f %6.2f %6.2f %3hu %s  %s %8lu\n\r",
                  spicture(tptr->data->name,20,temp),
                  netGetMachType(tptr->data->machine,temp3),tptr->data->l1,
                  tptr->data->l2,tptr->data->l3,varUsers(tptr->data),
                  (varConsole(tptr->data) ? "YES" : "NO "),
                  fmt_time(tptr->data->logtime,temp2),tptr->data->tmp);
     }
}


#ifdef __STDC__
void treeDump(FILE *outf,TREE *tptr,u_char opcode,int *i)
#else
void treeDump(outf,tptr,opcode,i)
FILE *outf;
TREE *tptr;
u_char opcode;
int *i;
#endif
{
     if (((opcode==OP_DISPLIST) && (*i>=MAXLSTLEN)) || (tptr==NULL)) return;
     if (tptr->left!=NULL) treeDump(outf,tptr->left,opcode,i);
     if ((opcode==OP_GETLIST) ||
         ((opcode==OP_GETDOWN) && (((time(NULL)-tptr->data->logtime)>=15*60)) ||
         ((opcode==OP_DISPLIST) &&
          ((time(NULL)-tptr->data->logtime)<15*60) && (*i<MAXLSTLEN)))) {
          treePrintHost(outf,tptr);
          ++(*i);
     }
     if (tptr->right!=NULL) treeDump(outf,tptr->right,opcode,i);
     free(tptr);
}


#ifdef __STDC__
void treePrintList(FILE *outf,u_char machtype,u_char modeltype,u_char opcode)
#else
void treePrintList(outf,machtype,modeltype,opcode)
FILE *outf;
u_char machtype,modeltype,opcode;
#endif
{
     long t;
     int i;

     time(&t);
     treeBuild(machtype,modeltype);
     fprintf(outf,"The current time is: %s\n\r",ctime(&t));
     fprintf(outf,"Host name            Type    Load Averages     Ses Disp Last Report   /tmp    \n\r");
     fprintf(outf,"-------------------- ---- -------------------- --- ---- ------------- --------\n\r");
     fflush(outf);
     i=0;
     treeDump(outf,theTree,opcode,&i);
     fprintf(outf,"\n\r");
     fflush(outf);
}


#ifdef __STDC__
void treePrintHostWithModels(FILE *outf,TREE *tptr)
#else
void treePrintHostWithModels(outf,tptr)
FILE *outf;
TREE *tptr;
#endif
{
     char temp[132],temp2[132],temp3[132];

     if ((tptr->data->logtime==0) || ((time(NULL)-tptr->data->logtime)>=DOWNTIME*60)) {
          fprintf(outf,"%19s %-4s%-15s ------ ------ ------ --- ---  --------\n\r",
                  spicture(tptr->data->name,19,temp),
                  netGetMachType(tptr->data->machine,temp3),
                  netGetModelType(tptr->data->model,temp2));
     }
     else {
          fprintf(outf,"%19s %-4s%-15s %6.2f %6.2f %6.2f %3d %s  %8lu\n\r",
                  spicture(tptr->data->name,19,temp),
                  netGetMachType(tptr->data->machine,temp3),
                  netGetModelType(tptr->data->model,temp2),
                  tptr->data->l1,tptr->data->l2,tptr->data->l3,varUsers(tptr->data),
                  (varConsole(tptr->data) ? "YES" : "NO "),tptr->data->tmp);
     }
}


#ifdef __STDC__
void treeDumpWithModels(FILE *outf,TREE *tptr,u_char opcode,int *i)
#else
void treeDumpWithModels(outf,tptr,opcode,i)
FILE *outf;
TREE *tptr;
u_char opcode;
int *i;
#endif
{
     if (((opcode==OP_DISPMLIST) && (*i>=MAXLSTLEN)) || (tptr==NULL)) return;
     if (tptr->left!=NULL) treeDumpWithModels(outf,tptr->left,opcode,i);
     if ((opcode==OP_GETMLIST) ||
         ((opcode==OP_GETMDOWN) && (((time(NULL)-tptr->data->logtime)>=15*60)) ||
         ((opcode==OP_DISPMLIST) &&
          ((time(NULL)-tptr->data->logtime)<15*60) && (*i<MAXLSTLEN)))) {
          treePrintHostWithModels(outf,tptr);
          ++(*i);
     }
     if (tptr->right!=NULL) treeDumpWithModels(outf,tptr->right,opcode,i);
     free(tptr);
}


#ifdef __STDC__
void treePrintListWithModels(FILE *outf,u_char machtype,u_char modeltype,u_char opcode)
#else
void treePrintListWithModels(outf,machtype,modeltype,opcode)
FILE *outf;
u_char machtype,modeltype,opcode;
#endif
{
     long t;
     int i;

     time(&t);
     treeBuild(machtype,modeltype);
     fprintf(outf,"The current time is: %s\n\r",ctime(&t));
     fprintf(outf,"Host name           Type and Model         Load Averages     Ses Disp /tmp    \n");
     fprintf(outf,"------------------- ------------------- -------------------- --- ---- --------\n\r");
     fflush(outf);
     i=0;
     treeDumpWithModels(outf,theTree,opcode,&i);
     fprintf(outf,"\n\r");
     fflush(outf);
}
